import 'package:exide_crr/res/app_color_text_size.dart';
import 'package:flutter/material.dart';
import 'package:google_fonts/google_fonts.dart';

class CircularAvatar extends StatelessWidget {
  final bool enableGrad;
  final String assetName;
  final double? width;
  final double? height;
  final bool showBadge;
  final bool isDrawerEnabled;
  final Function() onTap;
  final String badgeCount;
  const CircularAvatar({
    this.enableGrad = false,
    required this.assetName,
    this.height,
    this.width,
    this.showBadge = true,
    this.isDrawerEnabled = true,
    this.badgeCount = "",
    required this.onTap,
    super.key,
  });

  @override
  Widget build(BuildContext context) {
    return GestureDetector(
      onTap: () => onTap(),
      child: Container(
        alignment: Alignment.center,
        width: width ?? 32,
        height: height ?? 32,
        decoration: BoxDecoration(
          color: const Color(0XFFFFFFFF),
          border: Border.all(
            color:
                enableGrad ? const Color(0xFF5C4D57) : const Color(0xFFE2DBEC),
          ),
          shape: BoxShape.circle,
          gradient: enableGrad
              ? const LinearGradient(
                  colors: [
                    Color(0xFF47263D),
                    Color(0xFF3A2B49),
                  ],
                )
              : null,
        ),
        child: isDrawerEnabled
            ? Badge(
                isLabelVisible: badgeCount.isNotEmpty && badgeCount != "0",
                largeSize: 12,
                label: Text(
                  badgeCount,
                  style: GoogleFonts.figtree(
                    fontWeight: FontWeight.w600,
                    fontSize: 6,
                  ),
                ),
                child: Padding(
                  padding: const EdgeInsets.all(2.0),
                  child: Image.asset(
                    assetName,
                    color: enableGrad ? null : AppColor.black,
                  ),
                ),
              )
            : const Icon(
                Icons.arrow_back_rounded,
                color: Colors.white,
                size: 16,
              ),
      ),
    );
  }
}
