/* Created by Nitesh Kumar Verma on 05/28/24 */

import 'package:exide_crr/constants/enums.dart';
import 'package:exide_crr/constants/image_constants.dart';
import 'package:exide_crr/res/app_color_text_size.dart';
import 'package:exide_crr/utils/res/styles.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';
import 'package:outline_gradient_button/outline_gradient_button.dart';

// ignore: must_be_immutable
class CustomTextField extends StatelessWidget {
  CustomTextField({
    Key? key,
    required this.textFieldText,
    required this.hintText,
    this.isFocus = false,
    this.isSuccess = true,
    this.isPasswordField = false,
    this.isPasswordFieldEmpty = false,
    this.isPasswordHide = false,
    this.onSuffixIconClicked,
    this.focusNode,
    this.textEditingController,
    this.onSubmit,
    this.onChanged,
    this.onValidator,
    this.inputType = "text",
    this.textInputType = TextInputType.text,
    this.inputAction = TextInputAction.next,
    this.noOfLines = 1,
    this.maxLength = 60,
    this.topMarginTextField = 4,
    this.topMarginTextFieldTitle = 0,
    this.bottomMarginTextFieldTitle = 0,
    this.leftMarginTextFieldTitle = 0,
    this.rightMarginTextFieldTitle = 0,
    this.mandatoryText,
    this.readOnly = false,
    this.autoFocus = false,
  }) : super(key: key);

  //Text field title text
  final String textFieldText;

  //Text field hint text
  String hintText;

  //Text field focus status
  bool isFocus;

  /*  Variable define that, After API request
   * API response either success or not.*/
  bool isSuccess;

  //Define text field either password field or a general field
  bool isPasswordField;

  bool isPasswordFieldEmpty;

  ///If text field is a [isPasswordField] then it checks password
  ///filed hide unHide.
  bool isPasswordHide = false;

  /// request for autofocus when pages lodes
  bool autoFocus = false;

  /// max lines to define height of text field
  int noOfLines = 1;

  int maxLength = 60;

  /// top margin for text field
  double topMarginTextField = 4;

  /// top margin for text field title
  double topMarginTextFieldTitle = 12;

  /// bottom margin for text field title
  double bottomMarginTextFieldTitle = 0;

  /// bottom margin for text field title
  double leftMarginTextFieldTitle = 24;

  /// bottom margin for text field title
  double rightMarginTextFieldTitle = 24;

  //Text field focus node
  FocusNode? focusNode;

  //Text field edit controller
  TextEditingController? textEditingController;

  /// The callback called when the 'onSubmit' called.
  void Function(String value)? onSubmit;

  /// The callback called when the a new character enter into text field
  void Function(String value)? onChanged;

  /// The callback called when the click on password hide unHide button clicked
  VoidCallback? onSuffixIconClicked;

  /// The callback called when the validator check the validation
  FormFieldValidator<String>? onValidator;

  /// Text input type
  /// [TextInputType.text], [TextInputType.emailAddress], [TextInputType.number]
  /// [TextInputType.multiline] etc.
  TextInputType textInputType;

  String inputType = InputType.text.name;

  /// Text input action
  /// [TextInputAction.go], [TextInputAction.next], [TextInputAction.done] etc.
  TextInputAction inputAction;

  String? mandatoryText;
  bool? readOnly = false;

  @override
  Widget build(BuildContext context) {
    return Padding(
      padding: EdgeInsets.only(
          top: topMarginTextFieldTitle,
          bottom: bottomMarginTextFieldTitle,
          left: leftMarginTextFieldTitle,
          right: rightMarginTextFieldTitle),
      child: Column(
        crossAxisAlignment: CrossAxisAlignment.start,
        mainAxisAlignment: MainAxisAlignment.start,
        children: [
          ///Text filed title
          if (textFieldText.isNotEmpty)
            Padding(
              padding: EdgeInsets.symmetric(horizontal: 4.w),
              child: Row(
                mainAxisSize: MainAxisSize.min,
                mainAxisAlignment: MainAxisAlignment.start,
                children: [
                  Text(textFieldText, style: Styles.textFieldTitleStyle),
                  if (mandatoryText != null && mandatoryText!.isNotEmpty)
                    Text(mandatoryText!, style: Styles.mandatoryTitleStyle),
                ],
              ),
            ),
          if (textFieldText.isNotEmpty)
            SizedBox(
              height: 8.h,
            ),

          Container(
            margin: EdgeInsets.only(top: topMarginTextField),
            decoration: BoxDecoration(
                borderRadius: BorderRadius.circular(10),
                color: AppColor.lightWhite

                ///Check API success or not
                // border: Border.all(
                //   color: !isSuccess
                //       ? AppColor.primaryRedColor
                //       : AppColor.transparent,
                //   width: 2,
                // )
                ),
            child: TextFormField(
              autofocus: autoFocus,
              readOnly: readOnly!,
              focusNode: focusNode,
              controller: textEditingController,
              keyboardType: textInputType,
              cursorColor: AppColor.greyColor,

              cursorHeight: 20,
              maxLines: noOfLines,
              textInputAction: inputAction,
              autocorrect: false,
              inputFormatters: inputType == InputType.number.name
                  ? [
                      FilteringTextInputFormatter.digitsOnly,
                      LengthLimitingTextInputFormatter(maxLength),
                      FilteringTextInputFormatter.deny(RegExp(r'\s')),
                      FilteringTextInputFormatter.deny(
                          RegExp(r'[!@#$%^&*(),.?":{}|<>]'))
                    ]
                  : inputType == InputType.spaceNotAccepted.name
                      ? [FilteringTextInputFormatter.deny(RegExp(r'\s'))]
                      : inputType == InputType.specialCharacterNotAccepted.name
                          ? [
                              FilteringTextInputFormatter.deny(
                                  RegExp(r'[!@#$%^&*(),.?":{}|<>]'))
                            ]
                          : null,

              ///Check password filed or general
              obscureText: isPasswordField ? isPasswordHide : false,
              style: isSuccess
                  ? Styles.textFieldStyle
                  : Styles.textFieldErrorStyle,
              decoration: InputDecoration(
                contentPadding:
                    const EdgeInsets.symmetric(vertical: 16, horizontal: 16),
                hintStyle: Styles.hintTextFieldStyle,
                hintText: hintText,

                /// check focus on for selected text filed
                fillColor: isFocus ? AppColor.whiteColor : AppColor.whiteColor,
                filled: false,
                border: OutlineInputBorder(
                  borderRadius: BorderRadius.circular(10.h),
                  borderSide: BorderSide.none,
                ),
                enabledBorder: OutlineInputBorder(
                  borderRadius: BorderRadius.circular(10.h),
                  borderSide: BorderSide.none,
                ),
                focusedBorder: OutlineInputBorder(
                  borderRadius: BorderRadius.circular(10.h),
                  borderSide: BorderSide.none,
                ),
                // errorBorder: CustomBorder.enabledBorder,
                // focusedErrorBorder: CustomBorder.focusedErrorBorder,
                // errorStyle: TextStyle(
                //   color: AppColor.redLight,
                //   fontSize: 12,
                // ),

                /// Visible if password field is [True]
                suffixIcon: isPasswordFieldEmpty
                    ? GestureDetector(
                        onTap: onSuffixIconClicked,
                        child: Container(
                          margin: const EdgeInsets.symmetric(horizontal: 12),
                          child: Image.asset(
                            isPasswordHide
                                ? ImageConstants.passwordHideEye
                                : ImageConstants.passwordUnhideEye,
                            height: 12,
                            width: 12,
                            color: isSuccess
                                ? AppColor.greyColor
                                : AppColor.errorTextColor,
                          ),
                        ),
                      )
                    : null,

                /// Visible if password field is [True]
                //   prefixIcon: isPasswordField ? Icon(
                //     Icons.lock,
                //     size: 16,
                //     color: AppColor.greyColor,
                //   ) : null,
              ),
              onFieldSubmitted: onSubmit,
              onChanged: onChanged,
              validator: onValidator,
            ),
          ),
        ],
      ),
    );
  }
}
