import 'package:exide_crr/res/app_color_text_size.dart';
import 'package:flutter/material.dart';
import 'package:google_fonts/google_fonts.dart';

class LanguageSelectionButton extends StatelessWidget {
  final String language;
  final String? selectedLanguage;
  final ValueChanged<String?> onChanged;

  const LanguageSelectionButton({
    super.key,
    required this.language,
    required this.selectedLanguage,
    required this.onChanged,
  });

  @override
  Widget build(BuildContext context) {
    bool isSelected = selectedLanguage == language;

    return Container(
      padding: const EdgeInsets.all(10),
      decoration: BoxDecoration(
        borderRadius: const BorderRadius.all(
          Radius.circular(12),
        ),
        border: Border.all(
            color: isSelected
                ? AppColor.peach
                : AppColor.lightGreyBorderColor),
        color: isSelected ? AppColor.peach : AppColor.white,
      ),
      child: Row(
        mainAxisAlignment: MainAxisAlignment.spaceBetween,
        children: [
          Row(
            children: [
              languageIcon(language),
              const SizedBox(width: 12),
              Text(
                language == 'English' ? 'English' : 'हिन्दी',
                style: GoogleFonts.beVietnamPro(
                  fontSize: TextSize.textMediumSize,
                  fontWeight: FontWeight.w500,
                  color: AppColor.black,
                ),
              )
            ],
          ),
          Radio<String>(
            fillColor:
                MaterialStateProperty.resolveWith((Set<MaterialState> states) {
              if (states.contains(MaterialState.selected)) {
                return AppColor.darkRed;
              }
              return AppColor.darkGray;
            }),
            value: language,
            groupValue: selectedLanguage,
            onChanged: onChanged,
          ),
        ],
      ),
    );
  }

  Widget languageIcon(String language) {
    return Container(
      alignment: Alignment.center,
      height: 40,
      width: 40,
      decoration: const BoxDecoration(
          borderRadius: BorderRadius.all(Radius.circular(50)),
          color: AppColor.darkPeach),
      child: Text(language == 'English' ? 'A' : 'अ',
          style: GoogleFonts.beVietnamPro(
            fontSize: 20,
            fontWeight: FontWeight.w600,
            color: AppColor.darkRed,
          )),
    );
  }
}
