import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:exide_crr/data/firestore_service.dart';
import 'package:exide_crr/model/condition_data.dart';
import 'package:exide_crr/utils/helper_class.dart';

class FirebaseService {


  /// Initialize Firebase
  final FirebaseFirestore db = FireStoreService().fireStore;
  // static FirebaseFirestore db = FirebaseFirestore.instance;
  static final FirebaseService instance = FirebaseService._internal();
  FirebaseService._internal();

  Future<String> addNewRecord({collectionName, body}) async {
    var query = db.collection(collectionName);
    String docId = "";

    if(body != null) {
      var snapshot = await query.add(body);
      docId = snapshot.id;
    }

    return docId;
  }

  Future<void> updateRecord({collectionName, body, docId}) async {
    var query = db.collection(collectionName);

    if(docId != null && body != null) {
      await query.doc(docId).update(body);
    }
  }

  Future<void> deleteRecord({collectionName, docId}) async {
    var query = db.collection(collectionName);
    if(docId != null) {
      await query.doc(docId).delete();
    }
  }

  Future<Map<String, dynamic>> getAllRecordMap({required String collectionName, List<ConditionData>? conditions}) async {
    var query = db.collection(collectionName);
    if(conditions != null) {
      for (var element in conditions) {
        query.where(element.key!, isEqualTo: element.value!);
      }
    }

    // query.where("id", isEqualTo: "hello").where("p", isEqualTo: "uhfe");
    HelperClass.printWrapped("query====${query.toString()}");

    var snapshot = await query.get();
    var data = snapshot.docs.first.data();

    HelperClass.printWrapped("data====${data.toString()}");

    return data;
  }

  Future<List<Map<String, dynamic>>> getAllRecordList({required String collectionName, List<ConditionData>? conditions}) async {
    var query = db.collection(collectionName);
    if(conditions != null) {
      for (var element in conditions) {
        query.where(element.key!, isEqualTo: element.value!);
      }
    }

    // query.where("id", isEqualTo: "hello").where("p", isEqualTo: "uhfe");
    HelperClass.printWrapped("query====${query.toString()}");

    var snapshot = await query.get();
    var data = snapshot.docs.map((e) => e.data()).toList();

    HelperClass.printWrapped("data====${data.toString()}");

    return data;
  }

  Future<Map<String, dynamic>?> getSingleRecord({required String collectionName, required String docId, List<ConditionData>? conditions}) async {
    var query = db.collection(collectionName);
    if(conditions != null) {
      for (var element in conditions) {
        query.where(element.key!, isEqualTo: element.value!);
      }
    }
    HelperClass.printWrapped("query====${query.toString()}");

    var snapshot = await query.doc(docId).get();
    var data = snapshot.data();

    return data;
  }
}