import 'dart:async';
import 'dart:convert';
import 'dart:developer';
import 'package:exide_crr/serverRequest/all_request.dart';
import 'package:exide_crr/utils/custom_toast.dart';
import 'package:exide_crr/utils/helper_class.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:http/http.dart' as http;

import '../serverRequest/all_request.dart';

class PreferenceHelper {
  static const String accessToken = 'access_token';
  static const String refreshToken = 'refresh_token';
  static const String adminId = 'admin_id';
  static const String driverId = 'driver_id';
  static const String studentId = 'student_id';
  static const String roleId = 'role_id';
  static const String firstName = 'first_name';
  static const String lastName = 'last_name';
  static const String email = 'email';
  static const String campusId = 'campus_id';
  static const String campusName = 'campus_name';
  static const String campusLatitude = 'campus_latitude';
  static const String campusLongitude = 'campus_longitude';
  static const String selectedLanguage = 'selected_language';


  static const String lastRefresh = 'last_refresh';
  static const int refreshInterval = 5 * 100 * 1000;

  static const String accessExpiryTime = "access_expiry_time";
  static const String refreshExpiryTime = "refresh_expiry_time";

  static Future<SharedPreferences> getInstance() async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    return prefs;
  }

  static Future saveSelectedLanguagePreferenceData(String language) async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    prefs.setString(selectedLanguage, language);
    HelperClass.printWrapped("language ===> $language");
  }

  static Future saveUserPreferenceData(
      Map<String, dynamic> userData) async {
    SharedPreferences prefs = await SharedPreferences.getInstance();

    log("RefreshToken====${userData[refreshToken]}");
    prefs.setString(accessToken, userData[accessToken]);
    prefs.setString(firstName, userData[firstName] ?? "");
    prefs.setString(lastName, userData[lastName] ?? "");
    prefs.setString(email, userData[email] ?? "");
    prefs.setString(adminId, "${userData[adminId]}" ?? "");
    prefs.setString(driverId, "${userData[driverId]}" ?? "");
    prefs.setString(studentId, "${userData[studentId]}" ?? "");
    prefs.setString(roleId, "${userData[roleId]}" ?? "");
    prefs.setString(refreshToken, userData[refreshToken]);
    prefs.setString(campusId, userData[campusId]);
    prefs.setString(campusName, userData[campusName]);
    prefs.setString(campusLatitude, userData[campusLatitude]);
    prefs.setString(campusLongitude, userData[campusLongitude]);
  }

  static Future saveProfilePreferenceData(userData) async {
    SharedPreferences prefs = await SharedPreferences.getInstance();

    prefs.setString(firstName, userData[firstName]);
    prefs.setString(lastName, userData[lastName]);
    prefs.setString(accessToken, userData[accessToken]);
  }

  static Future<void> clearUserPreferenceData() async {
    SharedPreferences prefs = await SharedPreferences.getInstance();

    // user
    prefs.remove(accessToken);
    prefs.remove(firstName);
    prefs.remove(lastName);
    prefs.remove(refreshToken);
    prefs.remove(email);
    prefs.remove(adminId);
    prefs.remove(driverId);
    prefs.remove(studentId);
    prefs.remove(roleId);
  }

  static Future<bool> setIntData(String key, int value) async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    return prefs.setInt(key, value);
  }

  static Future<int?> getIntData(String key) async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    return prefs.getInt(key);
  }

  static Future<bool> setBoolData(String key, bool value) async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    return prefs.setBool(key, value);
  }

  static Future<bool?> getBoolData(String key) async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    return prefs.getBool(key);
  }

  static Future<String?> getToken() async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    String? token;

    int? lastRefreshTime = prefs.getInt(lastRefresh) ?? 0;
    var now = DateTime.now().millisecondsSinceEpoch;
    log("LastRefresh=====$lastRefresh");
    log("Now=====$now");
    log("Condition1=====${(now - lastRefreshTime)}");
    log("Condition2=====${(now - lastRefreshTime) > refreshInterval}");
    if (((now - lastRefreshTime) > refreshInterval)) {
      token = await getNewToken();
      log("New token===$token");
    } else {
      log("old token===$token");
      token = prefs.getString(accessToken);
      log("old token1===$token");
    }
    return token;
  }

  static Future<bool> setToken(String token) async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    return prefs.setString(accessToken, token);
  }

  static Future<String?> getPreferenceData(String key) async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    return prefs.getString(key);
  }

  static Future<bool> setPreferenceData(String key, String value) async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    return prefs.setString(key, value);
  }

  static Future<void> clearPreferenceData(key) async {
    SharedPreferences prefs = await SharedPreferences.getInstance();

    prefs.remove(key);
  }

  static Future getNewToken() async {
    final String? refreshTok = await PreferenceHelper.getPreferenceData(refreshToken);
    final String? emailAddress = await PreferenceHelper.getPreferenceData(email);

    log("GetNewToken====>>>>${AllHttpRequest.apiUrl}token/refresh");
    log("RefreshToken====>>>>$refreshTok");
    // set up POST request arguments
    String url = "${AllHttpRequest.apiUrl??""}token/refresh";
    Map<String, String> headers = {
    "Accept":"application/json",
    "Content-Type":"application/json"
    };

    var body = {
      "refresh_token": "$refreshTok",
      "email": "$emailAddress"
    };

    var responseData = await http.post(Uri.parse(url), headers: headers, body: json.encode(body));
    log("${responseData.request!.headers}");
    log("${responseData.request}");
    log("${responseData.statusCode}");
    log('get Response body: ${responseData.body}');

    if (responseData.statusCode == 200) {
      // If the call to the server was successful, parse the JSON.
      var response = json.decode(responseData.body);
      log('Refresh_Token: ${responseData.body}');
      if (response['status_code'].toString() != "200") {
        CustomToast.showToastMessage('Your session has been expired. Please login again.');

        /// clear the preference data
        PreferenceHelper.clearUserPreferenceData();

        // navigatorKey.currentState?.push();

        //Your session has been expired. Please login again.

        /// navigate to login screen
        // Navigator.pushAndRemoveUntil(
        //     context,
        //     SlideRightRoute(
        //       page: MultiBlocProvider(
        //         providers: [
        //           BlocProvider(create: (_) => AuthenticationBloc(authenticationRepository: AuthenticationRepository()),),
        //           BlocProvider(create: (_) => PasswordVisibilityBloc(),),
        //           BlocProvider(create: (_) => CommonFocusListenerBloc(),)
        //         ],
        //         child: const LoginScreen(),
        //       ),
        //     ),
        //     ModalRoute.withName(LoginScreen.tag)
        // );

      } else {
        PreferenceHelper.setPreferenceData(accessToken, response[accessToken]);
        PreferenceHelper.setPreferenceData(refreshToken, response[refreshToken]);
        PreferenceHelper.setIntData(lastRefresh, DateTime.now().millisecondsSinceEpoch);

        return response[accessToken];
      }
    }
  }
}
