import 'dart:developer';
import 'dart:io';

import 'package:device_info_plus/device_info_plus.dart';
import 'package:exide_crr/res/app_color_text_size.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter_spinkit/flutter_spinkit.dart';
import 'package:intl/intl.dart';
import 'dart:math' as Math;

import 'package:url_launcher/url_launcher.dart';

class HelperClass {
  static dynamic isEmailAddressValid1(String email) {
    Pattern pattern =
        r'^(([^<>()[\]\\.,;:\s@\"]+(\.[^<>()[\]\\.,;:\s@\"]+)*)|(\".+\"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$';
    RegExp regex =  RegExp("$pattern");
    if(email != '') {
      if (!regex.hasMatch(email)) {
        return 'Enter your valid email address';
      } else {
        return null;
      }
    } else {
      return 'Enter your email address';
    }
  }

  static bool isEmailAddressValid(String email) {
    // bool emailValid = RegExp("[a-zA-Z0-9._-]+@[a-z]+\.[a-z]+").hasMatch(email);
    Pattern pattern =
        r'^(([^<>()[\]\\.,;:\s@\"]+(\.[^<>()[\]\\.,;:\s@\"]+)*)|(\".+\"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$';
    RegExp regex =  RegExp("$pattern");

    bool emailValid = regex.hasMatch(email);
    log("EmailValid===$emailValid");
    return emailValid;
  }

  static bool isMobileValid(String mobile) {
    RegExp exp = RegExp(
      r"^(?:[+0]9)?[0-9]{10}$",
    );
    return exp.hasMatch(mobile.trim());
  }

  static bool isValidUrl(String url) {
    return Uri.tryParse(url)?.isAbsolute ?? false;
  }

  static String replaceAll(String string) {
    return string.replaceAll(RegExp(r"\s+\b|\b\s"), "");
    // we trim to remove trailing white spaces
  }

  // Minimum eight characters, at least one letter, one number and one special character:
  static bool validatePassword(String value) {
    return RegExp(r"^(?=.*[A-Za-z])(?=.*\d)(?=.*[@$!%*#?&^=+-])[A-Za-z\d@$!%*#?&^=+-]{6,}$").hasMatch(value)
        ? true
        : false;
  }

  static String getDateFormatMonthNameWithTime(String currentDate) {
    var formatter = DateFormat('d MMM, yyyy HH:mm');
    String formatted = formatter.format(DateTime.parse(currentDate));
    return formatted;
  }

  static String getCurrentLocationDateTime() {
    var now = DateTime.now();
    var formatter = DateFormat('MM-dd-yyyy HH:mm');
    String formatted = formatter.format(now);
    return formatted;
  }

  static String getDateFormatMonthName(String currentDate) {
    var formatter = DateFormat('d MMM, yyyy');
    String formatted = formatter.format(DateTime.parse(currentDate));
    return formatted;
  }

  static String getTimeWithoutDateFormat() {
    var now = DateTime.now();
    var formatter = DateFormat('hh:mm a');
    String formatted = formatter.format(now);
    return formatted;
  }

  static String getTimeFromDurationFormat(Duration duration) {
    String currentDate = formatHHMMSS(duration.inSeconds);
    var formatter = DateFormat('mm:ss');
    String formatted = formatter.format(DateFormat('mm:ss').parse(currentDate));
    return formatted;
  }

  static String getTime(Duration duration) {
    String twoDigits(int n) => n.toString().padLeft(2, "0");
    String twoDigitMinutes = twoDigits(duration.inMinutes.remainder(60));
    String twoDigitSeconds = twoDigits(duration.inSeconds.remainder(60));
    return "$twoDigitMinutes:$twoDigitSeconds";
  }

  static String getDateWithTimeFormat(String currentDate) {
    var formatter = DateFormat('d MMM yyyy, HH:mm');
    String formatted = formatter.format(DateTime.parse(currentDate));
    return formatted;
  }

  static String getDateTimeWithDayNameFormat(String currentDate) {
    var formatter = DateFormat('EEEE, MMMM d,  yyyy    HH:mm a');
    String formatted = formatter.format(DateTime.parse(currentDate));
    return formatted;
  }

  static String getDateWithOutTimeFormat(String currentDate) {
    var formatter = DateFormat('d MMM yyyy');
    String formatted = formatter.format(DateTime.parse(currentDate));
    log("DateForm ==== $formatted");
    return formatted;
  }

  static String getDateWithTimeFormat1(String currentDate) {
    var formatter = DateFormat('dd-MM-yyyy hh:mm:ss a');
    String formatted = formatter.format(DateTime.parse(currentDate));
    log("DateForm ==== $formatted");
    return formatted;
  }

  static String getMonthYearFormat(String currentDate) {
    var formatter = DateFormat('MMM yyyy');
    String formatted = formatter.format(DateTime.parse(currentDate));
    return formatted;
  }

  static String getTimeFormatDay(String currentDate) {
    var formatter = DateFormat('EEEE, d MMM, yyyy');
    String formatted = formatter.format(DateTime.parse(currentDate));
    return formatted;
  }

  static String getTimeFormatForDisplay(String? currentDate) {
    if(currentDate == null) {
      return "";
    }
    var formatter = DateFormat('MM/dd/yyyy');
    String formatted = formatter.format(DateTime.parse(currentDate));
    return formatted;
  }

  static String formatHHMMSS(int seconds) {
    int hours = (seconds / 3600).truncate();
    seconds = (seconds % 3600).truncate();
    int minutes = (seconds / 60).truncate();

    String hoursStr = (hours).toString().padLeft(2, '0');
    String minutesStr = (minutes).toString().padLeft(2, '0');
    String secondsStr = (seconds % 60).toString().padLeft(2, '0');

    if (hours == 0) {
      return "$minutesStr:$secondsStr";
    }

    return "$hoursStr:$minutesStr:$secondsStr";
  }

  static DateTime parseDate(String date, String dateformat, {bool utc = false}) {
    return DateFormat(dateformat).parse(date, utc);
  }

  static String formatDate(DateTime date, String dateFormat) {
    return DateFormat(dateFormat).format(date);
  }

  static String getResourceName({required String fileUrl}) {
    try {
      return fileUrl.split("/")[4].split("?").first;
    } catch (e) {
      return fileUrl;
    }
  }

  static bool validateString(String? value) {
    Pattern pattern = r'^[A-Za-z _]*[A-Za-z][A-Za-z _]*$';
    RegExp regex = RegExp(pattern.toString());

    if (value != '' && value != null) {
      if (!regex.hasMatch(value)) {
        return false;
      } else {
        return true;
      }
    } else {
      return false;
    }
  }

  static showSnackBar(context, message) {
    return SnackBar(
      content: Text(
        '$message',
        style: const TextStyle(
            color: AppColor.whiteColor,
            fontWeight: FontWeight.w600,
            fontStyle: FontStyle.normal,
            fontSize: TextSize.bodySize),
      ),
      backgroundColor: AppColor.primaryColor,
      duration: const Duration(seconds: 2),
    );
  }

  static void progressLoader(BuildContext context) {
    showDialog(
      context: context,
      barrierColor: AppColor.greyColor.withOpacity(0.6),
      barrierDismissible: true,
      builder: (BuildContext context) {
        return Dialog(
          // backgroundColor: AppColor.primaryColor.withOpacity(0.9),
          // shape: RoundedRectangleBorder(borderRadius: BorderRadius.circular(12.0)),
          child: Container(
            width: 100,
            height: 100,
            decoration: BoxDecoration(
                color: AppColor.primaryColor.withOpacity(0.9),
                borderRadius: BorderRadius.circular(12)
            ),
            child: const Center(
              child: Column(
                crossAxisAlignment: CrossAxisAlignment.center,
                mainAxisAlignment: MainAxisAlignment.center,
                children: [
                  CircularProgressIndicator(
                    color: AppColor.whiteColor,
                    backgroundColor: AppColor.greyColor,
                    strokeWidth: 3,
                  ),

                  SizedBox(height: 12,),
                  Text("Loading...", style: TextStyle(fontSize: 12, color: AppColor.whiteColor),)
                ],
              ),
            ),
          ),
        );

        /*AlertDialog(
          title: const Text('Basic dialog title'),
          content: const Text('A dialog is a type of modal window that\n'
              'appears in front of app content to\n'
              'provide critical information, or prompt\n'
              'for a decision to be made.'),
          actions: <Widget>[
            TextButton(
              style: TextButton.styleFrom(
                textStyle: Theme.of(context).textTheme.labelLarge,
              ),
              child: const Text('Disable'),
              onPressed: () {
                Navigator.of(context).pop();
              },
            ),
            TextButton(
              style: TextButton.styleFrom(
                textStyle: Theme.of(context).textTheme.labelLarge,
              ),
              child: const Text('Enable'),
              onPressed: () {
                Navigator.of(context).pop();
              },
            ),
          ],
        );*/
      },
    );
  }

  static Future<String> getLiveLocationUrl(double latitude, double longitude) async {
    String url = '';
    String urlAppleMaps = '';
    if (Platform.isAndroid) {
      url = 'https://www.google.com/maps/search/?api=1&query=$latitude,$longitude';
      return url;
    } else {
      urlAppleMaps = 'https://maps.apple.com/?q=$latitude,$longitude';
      url = 'comgooglemaps://?saddr=&daddr=$latitude,$longitude&directionsmode=driving';

      if (await canLaunchUrl(Uri.parse(url))) {
        return url;
      } else if (await canLaunchUrl(Uri.parse(urlAppleMaps))) {
        return urlAppleMaps;
      } else {
        return '';
      }
    }
  }

  static String timeToFormat(int time) { // --> time in form of seconds
    final int hour = (time / 3600).floor();
    final int minute = ((time / 3600 - hour) * 60).floor();
    final int second = ((((time / 3600 - hour) * 60) - minute) * 60).floor();

    final String setTime = [
      if (hour > 0) hour.toString().padLeft(2, "0") else "00",
      minute.toString().padLeft(2, "0"),
      second.toString().padLeft(2, '0'),
    ].join(':');
    return setTime;
  }

  static String formatDurationInHhMmSs(Duration duration) {
    final HH = (duration.inHours).toString().padLeft(2, '0');
    final mm = (duration.inMinutes % 60).toString().padLeft(2, '0');
    final ss = (duration.inSeconds % 60).toString().padLeft(2, '0');

    return '$HH:$mm:$ss';
  }

  static Map<String, String> roundOffDistance({required double distance}) {
    Map<String, String> result = {"distanceWithUnit": "10 meters", "distance": "0"};
    log("IncomingDistance====$distance");
    try {
      int dis = distance.ceil();

      int divisor = dis
          .toString()
          .length > 3
          ? 100
          : dis > 300
          ? 50
          : dis
          .toString()
          .length > 2
          ? 10
          : 10;

      var roundDistance = (dis ~/ divisor) * divisor;

      log("Distance===$roundDistance");
      if (roundDistance < 1000) {
        result = {"distanceWithUnit": "$roundDistance meters", "distance": "$roundDistance"};
      } else {
        var distanceInKm = (roundDistance * 0.001).toStringAsFixed(1);
        result = {"distanceWithUnit": "$distanceInKm km", "distance": "$roundDistance"};
      }
    } catch(e) {
      log("roundOffDistanceException===$e");
    }

    return result;
  }

  static bool getTimeStamp(time) {
    final DateTime timeStamp = DateTime.fromMillisecondsSinceEpoch(time * 1000, isUtc: false);

    var formatter = DateFormat('dd-MM-yyyy HH:mm:ss');
    String formatted = formatter.format(timeStamp);

    final int time1 = DateTime.now().millisecondsSinceEpoch;
    final DateTime timeStamp1 = DateTime.fromMillisecondsSinceEpoch(time1, isUtc: false);
    String formatted1 = formatter.format(timeStamp1);

    log("old time === $formatted");
    log("New time === $formatted1");

    return time < time1;
  }


  static SpinKitThreeBounce spinKit = const SpinKitThreeBounce(
    color: Colors.white,
    size: 27,
  );

  static SpinKitCircle roundLoader = const SpinKitCircle(
    color: AppColor.primaryColor,
    size: 20,
  );

  static Future<String> getDeviceInfo() async {
    String deviceToken = "";

    DeviceInfoPlugin deviceInfo = DeviceInfoPlugin();

    if (kIsWeb) {
      WebBrowserInfo webBrowserInfo = await deviceInfo.webBrowserInfo;
      deviceToken = webBrowserInfo.userAgent!;
    } else if (Platform.isAndroid) {
      AndroidDeviceInfo androidInfo = await deviceInfo.androidInfo;
      deviceToken = androidInfo.fingerprint;
    } else if (Platform.isIOS) {
      IosDeviceInfo iosInfo = await deviceInfo.iosInfo;
      deviceToken = iosInfo.identifierForVendor!;
    }

    return deviceToken;
  }

  static void printWrapped(String text) {
    final pattern = RegExp('.{1,800}'); // 800 is the size of each chunk
    pattern.allMatches(text).forEach((match) => print(match.group(0)));
  }
}

// Exp====1689970061
// [log] refExp====1690056461
// [log] New time === 20-01-1970 13:26:10
// [log] New time === 20-01-1970 13:27:36