import 'package:exide_crr/appLocalization/language_key.dart';
import 'package:exide_crr/constants/app_images.dart';
import 'package:exide_crr/constants/enums.dart';
import 'package:exide_crr/controller/base_controller.dart';
import 'package:exide_crr/customWidget/button_widget.dart';
import 'package:exide_crr/customWidget/customOnboardScreen/common_onboard_screen.dart';
import 'package:exide_crr/customWidget/languageSelectionButton/language_selection_button.dart';
import 'package:exide_crr/firebaseServices/firebase_service.dart';
import 'package:exide_crr/model/condition_data.dart';
import 'package:exide_crr/preferences/app_shared_preferences.dart';
import 'package:exide_crr/res/app_color_text_size.dart';
import 'package:exide_crr/routes/routes_name.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:get/get_utils/src/extensions/internacionalization.dart';
import 'package:google_fonts/google_fonts.dart';

class LanguageSelectionScreen extends StatefulWidget {
  const LanguageSelectionScreen({super.key});

  @override
  State<LanguageSelectionScreen> createState() =>
      _LanguageSelectionScreenState();
}

class _LanguageSelectionScreenState extends State<LanguageSelectionScreen> {
  BaseController baseController = Get.find();

  @override
  Widget build(BuildContext context) {
    return CommonOnboardScreen(
      backgroundImageName: AppImages.backgroundImage,
      child: selectLanguageSection(),
    );
  }

  Widget selectLanguageSection() {
    return Padding(
      padding: const EdgeInsets.symmetric(horizontal: 16, vertical: 24),
      child: Column(
        children: [
          Text(selectLanguage.tr,
              style: GoogleFonts.beVietnamPro(
                fontSize: TextSize.textLargeSize,
                fontWeight: FontWeight.w500,
                color: AppColor.black,
              )),
          const SizedBox(height: 10),
          Text(selectLanguageDescription.tr,
              textAlign: TextAlign.center,
              style: GoogleFonts.beVietnamPro(
                fontSize: TextSize.textSmallSize,
                fontWeight: FontWeight.w400,
                color: AppColor.lightGrey,
              )),
          const SizedBox(height: 16),
          /// english button
          LanguageSelectionButton(
            language: LanguageType.English.name,
            selectedLanguage: baseController.selectedLanguage.value,
            onChanged: (String? value) async {
              baseController.selectedLanguage.value = value!;
              Get.updateLocale(const Locale("en", "US"));
            },
          ),
          const SizedBox(height: 20),
          /// hindi button
          LanguageSelectionButton(
            language: LanguageType.Hindi.name,
            selectedLanguage: baseController.selectedLanguage.value,
            onChanged: (String? value) {
              baseController.selectedLanguage.value = value!;
              Get.updateLocale(const Locale("hi", "IN"));
            },
          ),
          const Spacer(),
          CustomButton(
            buttonName: saveMyPreference.tr,
            onTap: () {
              PreferenceHelper.saveSelectedLanguagePreferenceData(baseController.selectedLanguage.value);
              Get.offNamed(RoutesName.loginScreen);
            },
            activeBackgroundColor: AppColor.darkRed,
            width: Get.width,
            isActive: baseController.selectedLanguage.value.isNotEmpty,
          ),
        ],
      ),
    );
  }
}
